#include "pixt.h"

extern Widget regionBtn[];     /* defined in menubar.c */
extern XImage *global_reg_xim; /* defined in menubar.c */
void highlightRegion();
void highlightCell();
void clearCell();
void fillRegion();
void hideRegion();
void resetRegion();
void copyRegion();

extern unsigned long CurrentColor;
static char *getline();

void regionCB(w,client,call)
	Widget w;
	int client;
	caddr_t call;
{
	Arg args[10];
	/*
	** the sensitivity of the push buttons calling this function is
	** being managed, so we can only get these region action functions
	** if the button was sensitive, which means we really have a region.
	*/
	switch (client) {
	case HIDE_R:
		RegionOp = NOOP;
		hideRegion(ActiveRegion);
		resetRegion(&ActiveRegion);
		if (DrawState == REGION)
			DrawState = NOOP;
		RegionFirst = True;
		break;
	case MARK_R:
		/* the user wants to mark a region, so let's set the
		** DrawState to Region and return, then any subsequent
		** button actions in the drawing area will result in the
		** region being selected
		*/
		RegionOp = MARK_R;
		XDefineCursor(display,win,regionCursor);
		if (ActiveRegion.width != 0 || ActiveRegion.height != 0) {
			hideRegion(ActiveRegion);
			resetRegion(&ActiveRegion);
		}
		DrawState = REGION;
		RegionFirst = True;
		break;
	case COPY_R:
		RegionOp = COPY_R;
		XDefineCursor(display,win,regionCursor);
		DrawState = REGION;
		break;
	case SAVE_R:
		/*
		printf("getting image from pixwin:x=%d\ty=%d\tw=%d\th=%d\n",
				ActiveRegion.col,
				ActiveRegion.row,
				ActiveRegion.width,
				ActiveRegion.height);
		*/

		global_reg_xim = XGetImage(display,pixwin,
					ActiveRegion.col,
					ActiveRegion.row,
					ActiveRegion.width,
					ActiveRegion.height,
					AllPlanes,ZPixmap);
		DrawState = REGION;
		RegionOp = SAVE_R;
		ImageType = PIX;
		ImageOp   = R_WRITE;
		XtSetArg(args[0],XmNdirMask,
			XmStringCreateLtoR("*.xpm",XmSTRING_DEFAULT_CHARSET));
		XtSetValues(fileDialog,args,1);
		XtManageChild(fileDialog);
		break;
	case READ_R:
		DrawState   = REGION;
		RegionOp    = READ_R;
		ImageType   = PIX;
		ImageOp     = R_READ;
		XtSetArg(args[0],XmNdirMask,
			XmStringCreateLtoR("*.xpm",XmSTRING_DEFAULT_CHARSET));
		XtSetValues(fileDialog,args,1);
		XtManageChild(fileDialog);
		break;
	case FILL_R:
		DrawState = REGION;
		RegionOp = FILL_R;
		fillRegion(ActiveRegion);
		break;

	}
}

void handleRegion(call,row,col)
	XmDrawingAreaCallbackStruct  *call;
	int row, col; 
{
	int r, c;
	Arg args[5];
	/*
	** i'm here because button1 has been clicked in a cell and 
	** RegionFirst is False, meaning that the one corner of the 
	** region has already been marked and this is the second.
	** the job here is to calculate the row,col of the start of this
	** region and then the width height...
	*/
	XDefineCursor(display,win,gridCursor);	
	RegionFirst = True;
	if (row == ActiveRegion.row) {
		ActiveRegion.height = 1;
		if (col == ActiveRegion.col) 
			ActiveRegion.width = 1;
		else if (col < ActiveRegion.col) {
			ActiveRegion.width = ActiveRegion.col - col;
			ActiveRegion.col = col;
		}
		else /* col > ActiveRegion.col */ 
			ActiveRegion.width = col - ActiveRegion.col;
	}
	else if (row < ActiveRegion.row) {
		ActiveRegion.height = ActiveRegion.row - row + 1;
		ActiveRegion.row = row;
		if (col == ActiveRegion.col)
			ActiveRegion.width = 1;
		else if (col < ActiveRegion.col) {
			ActiveRegion.width = ActiveRegion.col - col + 1;
			ActiveRegion.col = col;
		}
		else /* col > ActiveRegion.col */
			ActiveRegion.width = col - ActiveRegion.col + 1;
	}
	else /* r > ActiveRegion.row */ {
		ActiveRegion.height = row -ActiveRegion.row + 1;
		if (col == ActiveRegion.col)
			ActiveRegion.width = 1;
		else if (col < ActiveRegion.col) {
			ActiveRegion.width = ActiveRegion.col - col + 1;
			ActiveRegion.col = col;
		}
		else
			ActiveRegion.width = col - ActiveRegion.col + 1;
	}

	/* now that we have the region parameters in the global
	** ActiveRegion, we just need to highlight it and turn on
	** the region pushbuttons
	*/
	highlightRegion(ActiveRegion);
	XtSetArg(args[0],XmNsensitive,True);
	XtSetValues(regionBtn[COPY_R],args,1);
	XtSetValues(regionBtn[SAVE_R],args,1);
	XtSetValues(regionBtn[FILL_R],args,1);
	XtSetValues(regionBtn[HIDE_R],args,1);
	XtSetValues(regionBtn[CAPTURE_R],args,1);
	DrawState = NOOP;
}

void highlightCell(row,col)
	int row;
	int col;
{
	XDrawRectangle(display,win,highlightGC,
			ColSegs[col].x1,RowSegs[row].y1,
			CELLSIZE+1,CELLSIZE+1);
}

void highlightRegion(pr)
	PixRegion pr;
{
	int r, c;
	if (pr.width == 1) {
		if (pr.height == 1)
			XDrawRectangle(display,win,highlightGC,
				ColSegs[pr.col].x1,RowSegs[pr.row].y1,
				CELLSIZE+1,CELLSIZE+1);
		else 
			for (r=pr.row;r<pr.row+pr.height;r++)
				XDrawRectangle(display,win,highlightGC,
				ColSegs[pr.col].x1,RowSegs[r].y1,
				CELLSIZE+1,CELLSIZE+1);
	}
	else if (pr.height == 1) {
		for (c=pr.col;c<pr.col+pr.width;c++)
			XDrawRectangle(display,win,highlightGC,
			ColSegs[c].x1,RowSegs[pr.row].y1,
			CELLSIZE+1,CELLSIZE+1);
	}
	else {
		for (r=pr.row;r<pr.row+pr.height;r++)
			for (c=pr.col;c<pr.col+pr.width;c++)
				XDrawRectangle(display,win,highlightGC,
					ColSegs[c].x1,RowSegs[r].y1,
					CELLSIZE+1,CELLSIZE+1);
	}
}

void hideRegion(pr)
	PixRegion pr;
{
	int r, c;
	if (pr.width == 1) {
		if (pr.height == 1)
			XDrawRectangle(display,win,gridGC,
				ColSegs[pr.col].x1,RowSegs[pr.row].y1,
				CELLSIZE+1,CELLSIZE+1);
		else 
			for (r=pr.row;r<pr.row+pr.height;r++)
				XDrawRectangle(display,win,gridGC,
				ColSegs[pr.col].x1,RowSegs[r].y1,
				CELLSIZE+1,CELLSIZE+1);
	}
	else if (pr.height == 1) {
		for (c=pr.col;c<pr.col+pr.width;c++)
			XDrawRectangle(display,win,gridGC,
			ColSegs[c].x1,RowSegs[pr.row].y1,
			CELLSIZE+1,CELLSIZE+1);
	}
	else {
		for (r=pr.row;r<pr.row+pr.height;r++)
			for (c=pr.col;c<pr.col+pr.width;c++)
				XDrawRectangle(display,win,gridGC,
					ColSegs[c].x1,RowSegs[r].y1,
					CELLSIZE+1,CELLSIZE+1);
	}
}

void clearCell(row,col)
	int row; 
	int col;
{
	XClearArea(display,win,
		ColSegs[col].x1,RowSegs[row].y1,
		CELLSIZE,CELLSIZE,
		False);
	
}

void fillRegion(pr)
	PixRegion pr;
{
	int r, c;
	int ps;
	for (r=pr.row;r<pr.row+pr.height;r++)
		for (c=pr.col;c<pr.col+pr.width;c++) {
			ps = r*NumCols+c;
			PixColors[ps] = CurrentColor;
			XSetForeground(display,paintGC,CurrentColor);
			XFillRectangle(display,win,paintGC,
				ColSegs[c].x1+1,
				RowSegs[r].y1+1,
				CELLSIZE,CELLSIZE);
			XDrawPoint(display,pixwin,
				paintGC,c,r);
		}
}

void resetRegion(pr)
	PixRegion *pr;
{
	Arg args[2];
	pr->width = 0;
	pr->height = 0;
	XtSetArg(args[0],XmNsensitive,False);
	XtSetValues(regionBtn[COPY_R],args,1);
	XtSetValues(regionBtn[SAVE_R],args,1);
	XtSetValues(regionBtn[HIDE_R],args,1);
	XtSetValues(regionBtn[FILL_R],args,1);
	XtSetValues(regionBtn[CAPTURE_R],args,1);
}

void copyRegion(pr,row,col)
	PixRegion pr;
	int row, col;
{
	int height, width, ic, ir, r, c;
	int ps, pd;
	/*
	** make sure that we don't try to copy outside the grid:
	*/
	if (row+pr.height > NumRows) 
		height = NumRows - row - 1;
	else 
		height = pr.height;
	if (col+pr.width > NumCols)
		width = NumCols - col - 1;
	else
		width = pr.width;
	
	for (r=pr.row,ir=0;r<=pr.row+height;r++,ir++)
		for (c=pr.col,ic=0;c<=pr.col+width;c++,ic++) {
			ps = r*NumCols+c;
			pd = (row+ir)*NumCols+col+ic;
			PixColors[pd] = PixColors[ps];
			XSetForeground(display,paintGC,PixColors[ps]);
			XFillRectangle(display,win,paintGC,
				ColSegs[col+ic].x1+1,
				RowSegs[row+ir].y1+1,
				CELLSIZE,CELLSIZE);
			XDrawPoint(display,pixwin,paintGC,col+ic,row+ir);
		}
}

writeRegionPixmap(fname)
{
	/*
	** ActiveRegion is a global...
	*/
	int fd;
	int r, c, ps;
	int width, height;
	Pixmap tmpmap;
	int screen = DefaultScreen(display);
	tmpmap = XCreatePixmap(display,pixwin,
			ActiveRegion.width,ActiveRegion.height,
			DefaultDepth(display,screen));
	XPutImage(display,tmpmap,paintGC,global_reg_xim,
			0,0,0,0,
			ActiveRegion.width,
			ActiveRegion.height);
	r = XWritePixmapFile(display,cmap,fname,tmpmap,
			ActiveRegion.width,
			ActiveRegion.height);
	XFreePixmap(display,tmpmap);
	if (r != PixmapSuccess) 
		return(-1);
	else
		return(1);
}


readRegionPixmap(fname)
{
	int fd, width, height, num_read;
	int ret;
	int depth; 
	Pixmap tmpmap;
	int screen = DefaultScreen(display);
	FILE *fstream;
  	char linebuf[256] ;
  	char namebuf[256];
  	char type[256];
  	char name[256];
	int p;

	if ((fstream = fopen(fname, "r")) == NULL) {
		return(-1);
	}

	getline(linebuf,fstream);	
	if ((sscanf(linebuf, "#define %[^_]%s %d", namebuf,type,&p) != 3) 
      		|| ((strcmp("_format",type)) && (strcmp("_paxformat",type)))
      		|| (p != XPM_FORMAT)) {
    		return(-1);
	}
	else 
		strcpy(name,namebuf);

	getline(linebuf,fstream);
	if ((sscanf(linebuf, "#define %[^_]%s %d",namebuf,type,&width) != 3)
      		|| (strcmp(name,namebuf)) 
      		|| (strcmp("_width",type))) {
		return(-1);
	}
  	getline(linebuf,fstream);
  	if ((sscanf(linebuf, "#define %[^_]%s %d",namebuf,type,&height) != 3)
      		|| (strcmp(name,namebuf)) 
      		|| (strcmp("_height",type))) {
		return(-1);
	}
	fclose(fstream);
	/* end of code from xpm_read.c */
	depth = DefaultDepth(display,screen);
	ret = XReadPixmapFile(display,pixwin,cmap,fname,
			&width, &height,
			depth, &tmpmap);

	if (ret == PixmapSuccess)
		redrawRegionFromPixmap(tmpmap,width,height);
	else
		return(-1);	
}


redrawRegionFromPixmap(p,width,height)
	Pixmap p;
	int    width;
	int    height;
{
	int ps, row, col;
	XImage *xim;
	int r,c ;

	/* the width and height passed in are the w&h of the pixmap that
	** was just read using the GroupeBull xpm routine. these may or
	** may not be the same dimensions of the ActiveRegion...
	*/

	xim = XGetImage(display,p,0,0,width,height,AllPlanes,ZPixmap);
	
	if (width > ActiveRegion.width)
		width = ActiveRegion.width;
	if (height > ActiveRegion.height)
		height = ActiveRegion.height;

	
	for (row=ActiveRegion.row,r=0;row<ActiveRegion.row+height;r++,row++)
		for (col=ActiveRegion.col,c=0;
		col<ActiveRegion.col+width;c++,col++) {
			ps = row*NumCols+col;
			PixColors[ps] = XGetPixel(xim,c,r);
			XSetForeground(display,paintGC,PixColors[ps]);
			XFillRectangle(display,win,paintGC,
				ColSegs[col].x1+1,
				RowSegs[row].y1+1,
				CELLSIZE,CELLSIZE);
			
		}
	XPutImage(display,pixwin,paintGC,xim,0,0,
			ActiveRegion.col,ActiveRegion.row,
			width,height);
}


static char *
getline(s,pF)
     char * s ;
     FILE * pF ;
{
    s = fgets(s,256,pF);

    while (s) {
	int len = strlen(s);
	if (len && s[len-1] == '\015')
	    s[--len] = '\0';
	if (len==0) s = fgets(s,256,pF);
	else break;
    }
    return(s);
}
	    
